unit XMLIniFile;
(*
This file uses janXMLParser2.  Download janXMLParser2 at
http://www.torry.net/vcl/internet/html/jveasyxml.zip

This was created quickly so there may be some bugs.
croller@objectforce.com
*)

interface
uses
  classes,
  IniFiles,
  janXMLParser2;

type
  TXMLIniFile = class(TCustomIniFile)
  private
    fXMLDOM: TjanXMLParser2;
    function GetSection(Section: String; aWrite: boolean = false): TjanXMLNode2;
    function GetIdent(Sect: TjanXMLNode2; Ident: String; aWrite: boolean = false): TjanXMLNode2;
    function GetValue(aIdent: TjanXMLNode2; aValue: String; aWrite: boolean = false): TjanXMLNode2;
  protected
  public
    constructor Create(const FileName: string);
    destructor Destroy; override;

    function ReadString(const Section, Ident, Default: string): string; override;
    procedure WriteString(const Section, Ident, Value: String); override;
    procedure ReadSection(const Section: string; Strings: TStrings); override;
    procedure ReadSections(Strings: TStrings); override;
    procedure ReadSectionValues(const Section: string; Strings: TStrings); override;
    procedure EraseSection(const Section: string); override;
    procedure DeleteKey(const Section, Ident: String); override;
    procedure UpdateFile; override;
    procedure Clear; virtual;
    property  DOM: TjanXMLParser2 read fXMLDOM;

    // merge inifiles. use to migrate "old" ini files to XML inifiles
    class procedure MergeFiles(Source: TCustomIniFile; Dest: TXMLIniFile);
  end;


implementation
uses
  SysUtils;

{ TXMLIniFile }

constructor TXMLIniFile.Create(const FileName: string);
begin
  inherited Create(FileName);
  fXMLDOM := TjanXMLParser2.Create();
  fXMLDOM.name := 'XML';
  if FileExists(FileName) then
    fXMLDOM.LoadXML(FileName);
end;

destructor TXMLIniFile.Destroy;
begin
  // EZ: do we realy need UpdateFile here ?
  UpdateFile;
  fXMLDOM.Free;
  inherited;
end;

procedure TXMLIniFile.EraseSection(const Section: string);
var
  Sect: TjanXMLNode2;
begin
  Sect := GetSection(Section);
  if Sect <> nil then
    Sect.parentNode.deleteNode(Sect);

end;

procedure TXMLIniFile.ReadSection(const Section: string;
  Strings: TStrings);
var
  Sect: TjanXMLNode2;
  Lf: Integer;
begin
  Strings.Clear;
  Sect := GetSection(Section);
  if Sect <> nil then begin
    for Lf := 0 to Sect.nodes.Count-1 do
    begin
      Strings.Add(TjanXMLNode2(Sect.nodes.Items[Lf]).text);
    end;
  end;
end;

procedure TXMLIniFile.ReadSections(Strings: TStrings);
var
  Lf: Integer;
begin
  Strings.Clear;
  for Lf := 0 to fXMLDOM.nodes.Count-1 do
    Strings.Add(TjanXMLNode2(fXMLDOM.nodes.Items[Lf]).text);
end;

procedure TXMLIniFile.ReadSectionValues(const Section: string;
  Strings: TStrings);
var
  Sect: TjanXMLNode2;
  Ident, lValue: TjanXMLNode2;
  Lf: Integer;
begin
  Strings.Clear;
  Sect := GetSection(Section);
  if Sect <> nil then begin
    for Lf := 0 to Sect.nodes.Count-1 do
    begin
      Ident := TjanXMLNode2(Sect.nodes.Items[Lf]);
      lValue := TjanXMLNode2(Ident.nodes.Items[0]);
      Strings.Add(lValue.text);
    end;
  end;
end;

function TXMLIniFile.ReadString(const Section, Ident,
  Default: string): string;
var
  Sect: TjanXMLNode2;
  Data: TjanXMLNode2;
begin
  Result := '';
  Sect := GetSection(Section);
  if Sect <> nil then
  begin
    Data := GetIdent(Sect, Ident);
    if Data <> nil then
      Result := TjanXMLNode2(Data.nodes.Items[0]).text;
  end
end;

procedure TXMLIniFile.UpdateFile;
begin
  fXMLDOM.SaveXML(FileName);
end;

procedure TXMLIniFile.WriteString(const Section, Ident, Value: String);
var
  Sect: TjanXMLNode2;
  Id: TjanXMLNode2;
  Val: TjanXMLNode2;
begin
  Sect := GetSection(Section, true);
  Id := GetIdent(Sect, Ident, true);
  Val := GetValue(Id, Value, true);
end;

class procedure TXMLIniFile.MergeFiles(Source: TCustomIniFile; Dest: TXMLIniFile);
var
  Lf, Lfs: Integer;
  Sections, Section: TStrings;
begin
  Assert(Assigned(Source) and Assigned(Dest));

  Sections := TStringList.Create;
  Section := TStringList.Create;
  try
    Source.ReadSections(Sections);
    for Lf := 0 to Sections.Count-1 do begin
      Source.ReadSection(Sections[Lf], Section);
      for Lfs := 0 to Section.Count-1 do begin
        Dest.WriteString(Sections[Lf], Section[Lfs], Source.ReadString(Sections[Lf], Section[Lfs], ''));
      end;
    end;
  finally
    Sections.Free;
    Section.Free;
  end;
end;

procedure TXMLIniFile.DeleteKey(const Section, Ident: String);
var
  Sect: TjanXMLNode2;
  Id: TjanXMLNode2;
begin
  Sect := GetSection(Section);
  if Sect <> nil then
  begin
    Id := GetIdent(Sect, Ident);
    if Id <> nil then
    begin
      TjanXMLNode2(Id.nodes.Items[0]).text := '';
    end;
  end;
end;

function TXMLIniFile.GetSection(Section: String; aWrite: boolean = false): TjanXMLNode2;
var lCtr: integer;
		lData: string;
begin
  result := nil;
  for lCtr := 0 to fXMLDOM.nodes.Count - 1 do
  begin
    lData := TjanXMLNode2(fXMLDOM.nodes.Items[lCtr]).text;
    if (AnsiCompareText(lData, Section) = 0) then
    begin
      result :=  TjanXMLNode2(fXMLDOM.nodes.Items[lCtr]);
      break;
    end;
  end;
  if (result = nil) and aWrite then
  begin
    result := TjanXMLNode2.create();
    fXMLDOM.addNode(result);
    result.Name := 'Section';
    result.text := Section;
  end;
end;

function TXMLIniFile.GetIdent(Sect: TjanXMLNode2;
  Ident: String; aWrite: boolean = false): TjanXMLNode2;
var lCtr: integer;
		lData: string;
begin
  result := nil;
  for lCtr := 0 to Sect.nodes.Count - 1 do
  begin
    lData := TjanXMLNode2(Sect.nodes.Items[lCtr]).text;
    if (AnsiCompareText(lData, Ident) = 0) then
    begin
      result :=  TjanXMLNode2(Sect.nodes.Items[lCtr]);
      break;
    end;
  end;
  if (result = nil) and aWrite then
  begin
    result := TjanXMLNode2.create();
    Sect.addNode(result);
    result.text := Ident;
    result.name := 'Ident';
  end;
end;

function TXMLIniFile.GetValue(aIdent: TjanXMLNode2;
  aValue: String; aWrite: boolean = false): TjanXMLNode2;
var lCtr: integer;
		lData: string;
begin
  result := nil;
  for lCtr := 0 to aIdent.nodes.Count - 1 do
  begin
    lData := TjanXMLNode2(aIdent.nodes.Items[lCtr]).text;
    if (AnsiCompareText(lData, aValue) = 0) then
    begin
      result :=  TjanXMLNode2(aIdent.nodes.Items[lCtr]);
      break;
    end;
  end;
  if (result = nil) and aWrite then
  begin
    result := TjanXMLNode2.create();
    aIdent.addNode(result);
    result.text := aValue;
    result.name := 'Value';
  end;
end;

procedure TXMLIniFile.Clear;
begin
  fXMLDOM.xml := '';
end;

end.

